namespace ClipboardTool
{
    #region Imports

    using System;

    using IDataObject = System.Windows.Forms.IDataObject;
    using Clipboard = System.Windows.Forms.Clipboard;
    using TextWriter = System.IO.TextWriter;
    using TextReader = System.IO.TextReader;
    using Assembly = System.Reflection.Assembly;
    using CultureInfo = System.Globalization.CultureInfo;
    using Encoding = System.Text.Encoding;
    using StreamWriter = System.IO.StreamWriter;
    using StreamReader = System.IO.StreamReader;
    using AssemblyCopyrightAttribute = System.Reflection.AssemblyCopyrightAttribute;

    #endregion

    internal enum Operation { None, Copy, Paste }

	internal class Application
    {
        private Options _options;

        private Application(Options options)
        {
            _options = options;
        }

        internal static void Main(string[] args)
        {
            try
            {
                Options options = Options.Parse(new CommandLineReader(args));
                Application app = new Application(options);
                app.Main();
            }
            catch (Exception e)
            {
                Console.Error.WriteLine(e.Message);
                Environment.ExitCode = ExitCodes.Error;
            }
        }

        private void Main()
        {
            if (_options.ShowHelp)
            {
                ShowHelp();
                return;
            }

            IDataObjectHandler formatHandler = GetFormatHandler();
                
            //
            // Perform the copy or paste operation.
            //

            if (_options.Operation == Operation.Copy)
            {
                using (TextReader reader = GetReader())
                    Copy(reader, formatHandler);
            }
            else if (_options.Operation == Operation.Paste)
            {
                //
                // NOTE: If the writer is file-based then we don't delete 
                // the file if an error occurs duringthe paste operation.
                //

                using (TextWriter writer = GetWriter())
                    Paste(writer, formatHandler);
            }
            else
            {
                throw new ProgramException("A required clipboard operation (copy or paste) was not specified.");
            }
        }

        private TextReader GetReader()
        {
            if (_options.FileName.Length > 0)
            {
                Encoding encoding = GetEncoding(Encoding.Default);
                return new StreamReader(_options.FileName, encoding);
            }
            else
            {
                Encoding encoding = GetEncoding(null);

                return encoding == null ? Console.In :
                    new StreamReader(Console.OpenStandardInput(), encoding);
            }
        }

        private TextWriter GetWriter()
        {
            if (_options.FileName.Length > 0)
            {
                Encoding encoding = GetEncoding(Encoding.Default);
                return new StreamWriter(_options.FileName, false, encoding);
            }
            else
            {
                Encoding encoding = GetEncoding(null);

                return encoding == null ? Console.Out : 
                    new StreamWriter(Console.OpenStandardOutput(), encoding);
            }
        }

        private static void Copy(TextReader reader, IDataObjectHandler handler)
        {
            Clipboard.SetDataObject(handler.Copy(reader), true);
        }

        private static void Paste(TextWriter writer, IDataObjectHandler handler)
        {
            //
            // Get the clipboard data and check if the desired format
            // is available or not.
            //

            IDataObject dataObject = Clipboard.GetDataObject();
                
            if (dataObject == null)
            {
                Environment.ExitCode = ExitCodes.DataNotAvailable;
                return;
            }

            if (!handler.Paste(dataObject, writer))
            {
                Environment.ExitCode = ExitCodes.FormatNotAvailable;
            }
        }

        private IDataObjectHandler GetFormatHandler()
        {
            //
            // Translate format option to the corresponding clipboard 
            // format.
            //

            IDataObjectHandler handler;
            _options.DataFormat = _options.DataFormat.ToLower(CultureInfo.InvariantCulture);

            switch (_options.DataFormat)
            {
                case "text" : handler = new TextHandler(); break;
                case "html" : handler = new HtmlHandler(); break;
                case "rtf"  : handler = new RtfHandler();  break;

                default :
                {
                    throw new ProgramException(string.Format(
                        "'{0}' is not a known or accepted clipboard format.", 
                        _options.DataFormat));
                }
            }

            return handler;
        }

        private Encoding GetEncoding(Encoding @default)
        {
            if (_options.Encoding.Length > 0)
            {
                return Encoding.GetEncoding(_options.Encoding);
            }
            else
            {
                return @default;
            }
        }

        public static void ShowHelp()
        {
            WriteLogo();
            Console.WriteLine();
            Options.WriteUsage();
        }
    
        private static void WriteLogo()
        {
            WriteLogo(Console.Out);
        }

        private static void WriteLogo(TextWriter writer)
        {
            Assembly thisAssembly = typeof(Application).Assembly;
            writer.WriteLine("Clipboard Tool v{0}", thisAssembly.GetName().Version.ToString());

            AssemblyCopyrightAttribute copyrightAttribute = (AssemblyCopyrightAttribute) Attribute.GetCustomAttribute(thisAssembly, typeof(AssemblyCopyrightAttribute));

            if (copyrightAttribute != null)
            {
                string copyright = copyrightAttribute.Copyright.Trim();

                if (copyright.Length > 0)
                    writer.WriteLine(copyright);
            }

            writer.Write("Running on Microsoft (R) .NET Framework version ");
            writer.WriteLine(Environment.Version.ToString());
        }
    }

    internal sealed class ExitCodes
    {
        private ExitCodes() {}

        public static readonly int Error = -1;
        public static readonly int Success = 0;
        public static readonly int DataNotAvailable = 1;
        public static readonly int FormatNotAvailable = 2;
    }
}
