using System.Text;

namespace ClipboardTool
{
    #region Imports

    using System;

    using DataFormats = System.Windows.Forms.DataFormats;
    using StreamReader = System.IO.StreamReader;
    using TextWriter = System.IO.TextWriter;
    using Stream = System.IO.Stream;
    using Debug = System.Diagnostics.Debug;
    using BaseEncoding = System.Text.Encoding;

    #endregion

    internal sealed class Options
    {
        public Operation Operation = Operation.None;
        public string DataFormat = DataFormats.Text;
        public bool ShowHelp = false;
        public string FileName = string.Empty;
        public string Encoding = string.Empty;

        public static Options Parse(CommandLineReader reader)
        {
            Debug.Assert(reader != null);

            Options options = new Options();

            while (reader.Read() != null)
            {
                if (reader.Token is CommandLineName)
                {
                    string option = reader.Token.Text;

                    switch (option)
                    {
                        case "F" :
                        case "file" :
                        {
                            options.FileName = reader.ReadValue().Text;
                            break;
                        }

                        case "e" :
                        case "encoding" :
                        {
                            options.Encoding = reader.ReadValue().Text;
                            break;
                        }

                        case "?" :
                        case "help" :
                        {
                            options.ShowHelp = true;
                            break;
                        }

                        case "f" :
                        case "format" :
                        {
                            options.DataFormat = reader.ReadValue().Text;
                            break;
                        }

                        default :
                        {
                            throw new CommandLineException(string.Format(
                                "The option '{0}' is not valid.", option));
                        }
                    }
                }
                else
                {
                    string operation = reader.Token.Text;

                    if (options.Operation != Operation.None)
                    {
                        throw new CommandLineException("The clipboard operation must be specified once only.");
                    }

                    switch (operation)
                    {
                        case "c" :
                        case "copy" :
                        {
                            options.Operation = Operation.Copy;
                            break;
                        }

                        case "p" :
                        case "paste" :
                        {
                            options.Operation = Operation.Paste;
                            break;
                        }

                        default :
                        {
                            throw new CommandLineException(string.Format(
                                "'{0}' is not a valid clipboard operation.", operation));
                        }
                    }
                }
            }

            return options;
        }

        public static void WriteUsage()
        {
            WriteUsage(Console.Out);
        }

        public static void WriteUsage(TextWriter writer)
        {
            Debug.Assert(writer != null);

            Type thisType = typeof(Options);

            using (Stream stream = thisType.Assembly.GetManifestResourceStream(thisType, "Help.txt"))
            {
                System.Text.Encoding encoding = BaseEncoding.GetEncoding("Windows-1252");
                StreamReader reader = new StreamReader(stream, encoding);

                string text;

                try
                {
                    text = reader.ReadToEnd();
                }
                finally
                {
                    reader.Close();
                }

                BaseEncoding consoleEncoding = Console.Out.Encoding;
                BaseEncoding defaultEncoding = BaseEncoding.Default;

                text = string.Format(text, 
                    consoleEncoding.EncodingName,
                    consoleEncoding.WebName,
                    defaultEncoding.EncodingName,
                    defaultEncoding.WebName);

                writer.Write(text);
            }
        }
    }
}
